﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Fatture2010
{
    public partial class FormMain : Form
    {
        String ArchiveName = "";
        Boolean IsNewArchive = false;
        Binding RichTextBoxDescBinding = null;

        private String CompactPath(String Source)
        {
            String Result = String.Copy(Source);
            TextFormatFlags flags = TextFormatFlags.ModifyString | TextFormatFlags.PathEllipsis;
            TextRenderer.MeasureText(Result, SystemFonts.CaptionFont, new Size(this.Width - 350, 0), flags);
            return Result;
        }

        private Boolean InvoiceIsValid(DSInvoices.DTInvoicesRow drInvoice)
        {
            return (drInvoice.RowState == DataRowState.Unchanged ||
                    drInvoice.RowState == DataRowState.Added ||
                    drInvoice.RowState == DataRowState.Modified);
        }

        private DSInvoices.DTInvoicesRow GetCurrentInvoice()
        {
            if (dTInvoicesBindingSource.Current != null)
            {
                DSInvoices.DTInvoicesRow drInvoice = (DSInvoices.DTInvoicesRow)((DataRowView)dTInvoicesBindingSource.Current).Row;
                if (InvoiceIsValid(drInvoice))
                    return drInvoice;
            }

            return null;
        }

        private int GetMaxInvoiceNumber(int Year)
        {
            int MaxNum = 0;

            foreach (DSInvoices.DTInvoicesRow drInvoice in dSInvoices.DTInvoices.Rows)
                if (InvoiceIsValid(drInvoice))
                    if (drInvoice.Year == Year && drInvoice.Number > MaxNum)
                        MaxNum = drInvoice.Number;

            return MaxNum;
        }

        private void UpdateTotalFatt()
        {
            Decimal TotalFatt = new decimal(0);

            DSInvoices.DTInvoicesRow drInvoice;
            foreach (Object drRow in dTInvoicesBindingSource.List)
            {
                drInvoice = (DSInvoices.DTInvoicesRow)((DataRowView)drRow).Row;
                if (InvoiceIsValid(drInvoice))
                    TotalFatt = Decimal.Add(TotalFatt, drInvoice.Imponib);
            }

            toolStripTextBoxTotalFatt.Text = TotalFatt.ToString();
        }

        private void InitYearFilterComboBox()
        {
            Object FilterYear = toolStripComboBoxYear.SelectedItem;

            toolStripComboBoxYear.Items.Clear();
            toolStripComboBoxYear.Items.Add("Tutti");

            // Fill Year Filter ComboBox
            foreach (DSInvoices.DTInvoicesRow drInvoice in dSInvoices.DTInvoices.Rows)
                if (InvoiceIsValid(drInvoice))
                    if (toolStripComboBoxYear.Items.IndexOf(drInvoice.Year.ToString()) < 0)
                        toolStripComboBoxYear.Items.Add(drInvoice.Year.ToString());

            int FilterIndex = 0;
            if (FilterYear != null)
                FilterIndex = toolStripComboBoxYear.Items.IndexOf(FilterYear.ToString());

            if (FilterIndex > 0)
                toolStripComboBoxYear.SelectedIndex = FilterIndex;
            else
            {
                toolStripComboBoxYear.SelectedIndex = 0;
                dTInvoicesBindingSource.Filter = "";
                dTInvoicesBindingSource.ResetBindings(false);
            }

            UpdateTotalFatt();
        }

        private void SetArchiveName(String ArcName)
        {
            ArchiveName = ArcName;
            this.Text = "Fatture 2010 by Easy Target - " + CompactPath(ArchiveName);
        }

        private void NewArchive(String ArcName)
        {
            dSInvoices = new DSInvoices();
            dTInvoicesBindingSource.DataSource = dSInvoices;
            dTCustomersBindingSource.DataSource = dSInvoices;
            SetArchiveName(ArcName);
            IsNewArchive = true;
            toolStripBtnSave.Enabled = false;
            InitYearFilterComboBox();
        }

        private void ClearArchive(String ArcName)
        {
            dSInvoices.DTInvoices.Clear();
            SetArchiveName(ArcName);
            IsNewArchive = true;
            toolStripBtnSave.Enabled = false;
            InitYearFilterComboBox();
        }

        private void OpenArchive(String ArcName)
        {
            NewArchive(ArcName);
            dSInvoices.ReadXml(ArchiveName, XmlReadMode.IgnoreSchema);
            dSInvoices.AcceptChanges();
            IsNewArchive = false;
            toolStripBtnSave.Enabled = false;
            InitYearFilterComboBox();
            dTInvoicesBindingSource.MoveLast();
        }

        private void SaveArchive()
        {
            dSInvoices.AcceptChanges();
            dSInvoices.WriteXml(ArchiveName, XmlWriteMode.WriteSchema);
            toolStripBtnSave.Enabled = dSInvoices.HasChanges();
        }

        private Boolean CheckPendingUpdates()
        {
            if (dSInvoices.HasChanges())
            {
                if (MessageBox.Show("Le modifiche apportate all'archivio saranno perdute.\r\nSei sicuro di voler continuare?", "Archivio Modificato", MessageBoxButtons.YesNo) != DialogResult.Yes)
                    return false;
            }

            return true;
        }

        private void EnableEditing(Boolean InEdit)
        {
            dTInvoicesBindingSource.ResetBindings(false);
            toolStripBtnSave.Enabled = dSInvoices.HasChanges();

            dataGridInvoices.Enabled = !InEdit;
            bindingNavigatorDeleteItem.Enabled = !InEdit && (GetCurrentInvoice() != null);
            toolStripBtnEdit.Enabled = bindingNavigatorDeleteItem.Enabled;
            toolStripBtnUpdate.Enabled = InEdit;
            toolStripBtnCancel.Enabled = InEdit;
            toolStripBtnPrint.Enabled = bindingNavigatorDeleteItem.Enabled;
            panelData.Enabled = InEdit;

            bindingNavigatorAddNewItem.Enabled = !InEdit;
            bindingNavigatorDeleteItem.Enabled = !InEdit;
            
            SetDatePickerLimit(1990, 2050);

            // Enable DataSource Update of Rtf Manual Binding when Editing is active
            RichTextBoxDescBinding.DataSourceUpdateMode = InEdit ? DataSourceUpdateMode.OnValidation : DataSourceUpdateMode.Never;
        }

        private void SetDatePickerLimit(int MinYear, int MaxYear)
        {
            datePickInvoiceDate.MinDate = DateTime.Parse("01/01/1900"); // Ensure MinDate < MaxDate
            datePickInvoiceDate.MaxDate = DateTime.Parse("31/12/" + MaxYear.ToString());
            datePickInvoiceDate.MinDate = DateTime.Parse("01/01/" + MinYear.ToString());

            if (datePickInvoiceDate.Value <= datePickInvoiceDate.MinDate || datePickInvoiceDate.Value >= datePickInvoiceDate.MaxDate)
            {
                if (datePickInvoiceDate.MinDate > DateTime.Today)
                    datePickInvoiceDate.Value = datePickInvoiceDate.MinDate;
                else if (datePickInvoiceDate.MaxDate < DateTime.Today)
                    datePickInvoiceDate.Value = datePickInvoiceDate.MaxDate;
                else
                    datePickInvoiceDate.Value = DateTime.Today;
            }
        }

        public FormMain()
        {
            InitializeComponent();
            
            // Manual Binding to Rtf because it is not bindable at design time
            RichTextBoxDescBinding = richTextBoxDesc.DataBindings.Add("Rtf", dTInvoicesBindingSource, "Description", false, DataSourceUpdateMode.Never);

            // Restore Position
            SetBounds((int)Application.UserAppDataRegistry.GetValue("FormBoundsLeft", (Screen.PrimaryScreen.Bounds.Width-this.Width)/2),
                      (int)Application.UserAppDataRegistry.GetValue("FormBoundsTop", (Screen.PrimaryScreen.Bounds.Height - this.Height)/2),
                      (int)Application.UserAppDataRegistry.GetValue("FormBoundsWidth", this.Width),
                      (int)Application.UserAppDataRegistry.GetValue("FormBoundsHeight", this.Height));

            splitContainer.SplitterDistance = (int)Application.UserAppDataRegistry.GetValue("FormSplitterDistance", splitContainer.SplitterDistance);
        }

        private void FormMain_Resize(object sender, EventArgs e)
        {
            this.Text = "Fatture 2010 by Easy Target - " + CompactPath(ArchiveName);
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            String ArcName = Application.UserAppDataRegistry.GetValue("ArchiveName", "").ToString();
            
            if (ArcName != "" && System.IO.File.Exists(ArcName))
                OpenArchive(ArcName);
            else
                NewArchive("NuovoArchivioFatture.xml");
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !CheckPendingUpdates();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Save Actual Archive
            Application.UserAppDataRegistry.SetValue("ArchiveName", ArchiveName);

            // Save Position
            Application.UserAppDataRegistry.SetValue("FormBoundsLeft", this.Left);
            Application.UserAppDataRegistry.SetValue("FormBoundsTop", this.Top);
            Application.UserAppDataRegistry.SetValue("FormBoundsWidth", this.Width);
            Application.UserAppDataRegistry.SetValue("FormBoundsHeight", this.Height);
            Application.UserAppDataRegistry.SetValue("FormSplitterDistance", splitContainer.SplitterDistance);
        }


        private void toolStripBtnNew_Click(object sender, EventArgs e)
        {
            if (CheckPendingUpdates())
                if (MessageBox.Show("Sei sicuro di voler creare un nuovo Archivio?", "Nuovo Archivio", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    if (MessageBox.Show("Vuoi mantenere i Clienti e i Dati Personali attuali?", "Clienti e Dati", MessageBoxButtons.YesNo) == DialogResult.Yes)
                        ClearArchive("NuovoArchivioFatture.xml");
                    else
                    {
                        NewArchive("NuovoArchivioFatture.xml");
                        toolStripBtnPersData_Click(sender, e);
                        toolStripBtnClienti_Click(sender, e);
                    }
                }
        }

        private void toolStripBtnOpen_Click(object sender, EventArgs e)
        {
            if (CheckPendingUpdates())
            {
                OpenFileDialog openDlg = new OpenFileDialog();
                openDlg.FileName = "";
                
                if (openDlg.ShowDialog() == DialogResult.OK)
                    OpenArchive(openDlg.FileName);
            }
        }

        private void toolStripBtnSave_Click(object sender, EventArgs e)
        {
            if (IsNewArchive)
                toolStripBtnSaveAs_Click(sender, e);
            else
                SaveArchive();
        }

        private void toolStripBtnSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveDlg = new SaveFileDialog();
            saveDlg.FileName = ArchiveName;

            if (saveDlg.ShowDialog() != DialogResult.OK) return;

            SetArchiveName(saveDlg.FileName);
            IsNewArchive = false;

            SaveArchive();
        }

        private void toolStripBtnClienti_Click(object sender, EventArgs e)
        {
            FormCustomers frmCust = new FormCustomers();
            frmCust.MainDataSource = dSInvoices;
            
            if (frmCust.ShowDialog() != DialogResult.OK)
                dSInvoices.RejectChanges();

            toolStripBtnSave.Enabled = dSInvoices.HasChanges();
            dTCustomersBindingSource.ResetBindings(false);
        }

        private void toolStripBtnPersData_Click(object sender, EventArgs e)
        {
            FormPersonalData frmPers = new FormPersonalData();
            frmPers.MainDataSource = dSInvoices;

            if (frmPers.ShowDialog() != DialogResult.OK)
                dSInvoices.RejectChanges();

            toolStripBtnSave.Enabled = dSInvoices.HasChanges();
        }

        private void toolStripComboBoxYear_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (toolStripComboBoxYear.SelectedIndex <= 0)
                dTInvoicesBindingSource.Filter = "";
            else
                dTInvoicesBindingSource.Filter = "Year = " + toolStripComboBoxYear.SelectedItem;

            dTInvoicesBindingSource.ResetBindings(false);

            UpdateTotalFatt();
        }

        private void dataGridInvoices_SelectionChanged(object sender, EventArgs e)
        {
            if (dataGridInvoices.Enabled)
            {
                bindingNavigatorDeleteItem.Enabled = (GetCurrentInvoice() != null);
                toolStripBtnEdit.Enabled = bindingNavigatorDeleteItem.Enabled;
                toolStripBtnPrint.Enabled = bindingNavigatorDeleteItem.Enabled;
            }
        }

        private void toolStripBtnAdd_Click(object sender, EventArgs e)
        {
            EnableEditing(true);

            int Year = DateTime.Today.Year;
            int Numb = GetMaxInvoiceNumber(Year);

            // Ensure Values Change, so underlaying Fields are updated
            UpDownYear.Value = 2050;
            UpDownNumber.Value = 9999;

            // Set Valid Values
            UpDownYear.Value = Year;
            UpDownNumber.Value = Numb + 1;

            SetDatePickerLimit(Year, Year);
        }

        private void bindingNavigatorDeleteItem_Click(object sender, EventArgs e)
        {
            EnableEditing(false);
        }

        private void toolStripBtnEdit_Click(object sender, EventArgs e)
        {
            EnableEditing(true);
        }

        private void toolStripBtnUpdate_Click(object sender, EventArgs e)
        {
            if (comboBoxCustomer.Text != "")
            {
                // Dirty Patch: Ensure to write the CustomerName
                int selIdx = comboBoxCustomer.SelectedIndex;
                comboBoxCustomer.SelectedIndex = -1;
                comboBoxCustomer.SelectedIndex = selIdx;

                dTInvoicesBindingSource.EndEdit();
                EnableEditing(false);
                InitYearFilterComboBox();
            }
            else
                MessageBox.Show("Bisogna specificare un Cliente!");
        }

        private void toolStripBtnCancel_Click(object sender, EventArgs e)
        {
            dTInvoicesBindingSource.CancelEdit();
            EnableEditing(false);
        }

        private void toolStripBtnPrint_Click(object sender, EventArgs e)
        {
            FormReport frmReport = new FormReport();
            
            frmReport.MainDataSource = dSInvoices;
            frmReport.Invoice = GetCurrentInvoice();

            frmReport.ShowDialog();
        }

        private void UpDownYear_ValueChanged(object sender, EventArgs e)
        {
            int Year = (int)UpDownYear.Value;
            SetDatePickerLimit(Year, Year);
        }

        private void toolStripBtnBold_Click(object sender, EventArgs e)
        {
            if (richTextBoxDesc.SelectionFont != null)
            {
                FontStyle NewStyle = FontStyle.Regular;
                if (richTextBoxDesc.SelectionFont.Italic)
                    NewStyle = FontStyle.Italic;
                if (!richTextBoxDesc.SelectionFont.Bold)
                    NewStyle = NewStyle | FontStyle.Bold;

                System.Drawing.Font currentFont = richTextBoxDesc.SelectionFont;
                richTextBoxDesc.SelectionFont = new Font(currentFont.FontFamily, currentFont.Size, NewStyle);
            }
        }

        private void toolStripBtnItalic_Click(object sender, EventArgs e)
        {
            if (richTextBoxDesc.SelectionFont != null)
            {
                FontStyle NewStyle = FontStyle.Regular;
                if (richTextBoxDesc.SelectionFont.Bold) 
                    NewStyle = FontStyle.Bold;
                if (!richTextBoxDesc.SelectionFont.Italic)
                    NewStyle = NewStyle | FontStyle.Italic;

                System.Drawing.Font currentFont = richTextBoxDesc.SelectionFont;
                richTextBoxDesc.SelectionFont = new Font(currentFont.FontFamily, currentFont.Size, NewStyle);
            }
        }

        private void toolStripBtnBullets_Click(object sender, EventArgs e)
        {
            richTextBoxDesc.SelectionBullet = !richTextBoxDesc.SelectionBullet;
        }

        private void toolStripBtnIndent_Click(object sender, EventArgs e)
        {
            if (richTextBoxDesc.SelectionIndent < 480)
                richTextBoxDesc.SelectionIndent = richTextBoxDesc.SelectionIndent + 24;
        }

        private void toolStripBtnUnindent_Click(object sender, EventArgs e)
        {
            if (richTextBoxDesc.SelectionIndent > 0)
                richTextBoxDesc.SelectionIndent = richTextBoxDesc.SelectionIndent - 24;
        }

        private void toolStripBtnAlignLeft_Click(object sender, EventArgs e)
        {
            richTextBoxDesc.SelectionAlignment = HorizontalAlignment.Left;
        }

        private void toolStripBtnAlignRight_Click(object sender, EventArgs e)
        {
            richTextBoxDesc.SelectionAlignment = HorizontalAlignment.Right;
        }

    }
}
