﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Reporting.WinForms;

namespace Fatture2010
{
    public partial class FormReport : Form
    {
        public DSInvoices MainDataSource;
        public DSInvoices.DTInvoicesRow Invoice;

        public FormReport()
        {
            InitializeComponent();
            
            // Restore Position
            SetBounds((int)Application.UserAppDataRegistry.GetValue("ReportBoundsLeft", (Screen.PrimaryScreen.Bounds.Width - this.Width) / 2),
                      (int)Application.UserAppDataRegistry.GetValue("ReportBoundsTop", (Screen.PrimaryScreen.Bounds.Height - this.Height) / 2),
                      (int)Application.UserAppDataRegistry.GetValue("ReportBoundsWidth", this.Width),
                      (int)Application.UserAppDataRegistry.GetValue("ReportBoundsHeight", this.Height));
        }

        private void FormReport_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Save Position
            Application.UserAppDataRegistry.SetValue("ReportBoundsLeft", this.Left);
            Application.UserAppDataRegistry.SetValue("ReportBoundsTop", this.Top);
            Application.UserAppDataRegistry.SetValue("ReportBoundsWidth", this.Width);
            Application.UserAppDataRegistry.SetValue("ReportBoundsHeight", this.Height);
        }

        private void FormReport_Load(object sender, EventArgs e)
        {
            dTInvoicesBindingSource.DataSource = MainDataSource;
            dTCustomersBindingSource.DataSource = MainDataSource;
            dTPersonalDataBindingSource.DataSource = MainDataSource;

            dTInvoicesBindingSource.Filter = "Year=" + Invoice.Year.ToString() + " AND Number=" + Invoice.Number.ToString();
            dTCustomersBindingSource.Filter = "Name='" + Invoice.CustomerName +"'";

            this.reportViewer.LocalReport.DataSources.Add(new ReportDataSource("DTInvoices", dTInvoicesBindingSource));
            this.reportViewer.LocalReport.DataSources.Add(new ReportDataSource("DTCustomers", dTCustomersBindingSource));
            this.reportViewer.LocalReport.DataSources.Add(new ReportDataSource("DTPersonalData", dTPersonalDataBindingSource));

            this.reportViewer.LocalReport.SetParameters(new ReportParameter("AppPath", Application.StartupPath));

            // The Report TextBox don't understand RTF, but only HTML
            this.reportViewer.LocalReport.SetParameters(new ReportParameter("InvoiceHtmlDesc", Rtf2Html(Invoice.Description)));

            this.reportViewer.SetDisplayMode(DisplayMode.PrintLayout);
            this.reportViewer.RefreshReport();
        }


        // This function use a Very Dirty Approach!!!!
        private String Rtf2Html(String Rtf)
        {
            string[] LineSep = new string[] {"\r\n"};

            StringBuilder HtmlLines = new StringBuilder();

            int pos = Rtf.IndexOf("\\viewkind4\\uc1");
            Rtf = Rtf.Substring(pos+14);
            String[] RtfLines = Rtf.Split(LineSep, StringSplitOptions.None);
            
            HtmlLines.AppendLine("<BODY TOPMARGIN=0 LEFTMARGIN=0 MARGINHEIGHT=0 MARGINWIDTH=0>");
            HtmlLines.AppendLine("<font face=\"Tahoma\" size=\"3\">");
            HtmlLines.AppendLine("<p align=\"left\">");

            foreach (String rtfLine in RtfLines)
            {
                string Line = rtfLine;

                if (Line != "}")
                {
                    // Replace Special Chars
                    Line = Line.Replace("&", "&amp;");
                    Line = Line.Replace("<", "&lt;");
                    Line = Line.Replace(">", "&gt;");
                    Line = Line.Replace("\"", "&quot;");

                    // Handle Paragraph
                    Line = Line.Replace("\\pard\\qr", "</p><p align=\"right\">"); //style=\"text-align:right\">"); //
                    Line = Line.Replace("\\pard ", "</p>");
                    Line = Line.Replace("\\pard", "");
                    Line = Line.Replace("\\f0\\fs24", "");
                    Line = Line.Replace("\\par", "&nbsp;<br/>");

                    // Handle Bullets
                    Line = Line.Replace("{\\*\\pn\\pnlvlblt\\pnf1\\pnindent0{\\pntxtb\\'B7}}", "");
                    Line = Line.Replace("\\b{\\pntext\\f1\\'B7\\tab}", "&nbsp;&ordm;&nbsp;<b>");
                    Line = Line.Replace("\\i{\\pntext\\f1\\'B7\\tab}", "&nbsp;&ordm;&nbsp;<i>");
                    Line = Line.Replace("{\\pntext\\f1\\'B7\\tab}", "&nbsp;&ordm;&nbsp;");

                    // Handle Tabs e formatting spaces
                    Line = Line.Replace("\\tab", "<pre>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</pre>");

                    // Handle Indentation
                    pos = Line.IndexOf("\\li");
                    if (pos > -1)
                    {
                        char[] anyOf = { ' ', '\\' };
                        int aux = Line.IndexOfAny(anyOf, pos+3);
                        int offset = int.Parse(Line.Substring(pos+3, aux-pos-3));
                        String Spaces = "";

                        while (offset > 0)
                        {
                            Spaces = Spaces + "&nbsp;&nbsp;&nbsp;&nbsp;";
                            offset = offset - 360;
                        }
                        
                        String Beg = "";
                        if (pos > 0) Beg = Line.Substring(0, pos);

                        Line = "<pre>" + Spaces + "</pre>" + Beg + Line.Substring(aux);
                    }

                    // Handle Bold
                    Line = Line.Replace("\\b0", "</b>");
                    Line = Line.Replace("\\b ", "<b>");
                    Line = Line.Replace("\\b", "<b>");

                    // Handle Italic
                    Line = Line.Replace("\\i0", "</i>");
                    Line = Line.Replace("\\i ", "<i>");
                    Line = Line.Replace("\\i", "<i>");

                    // Handle Hexadecimal chars
                    String C = "";
                    pos = Line.IndexOf("\\'");
                    while (pos > -1)
                    {
                        C = Char.ConvertFromUtf32(Convert.ToInt32(Line.Substring(pos+2, 2), 16));
                        Line = Line.Substring(0, pos) + C + Line.Substring(pos+4);
                        pos = Line.IndexOf("\\'");
                    }

                    // Preserve Formatting Spaces
                    Line = Line.Replace("       ", "<pre>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</pre>");
                    Line = Line.Replace("      ", "<pre>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</pre>");
                    Line = Line.Replace("     ", "<pre>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</pre>");
                    Line = Line.Replace("    ", "<pre>&nbsp;&nbsp;&nbsp;&nbsp;</pre>");
                    Line = Line.Replace("   ", "<pre>&nbsp;&nbsp;&nbsp;</pre>");
                    Line = Line.Replace("  ", "<pre>&nbsp;&nbsp;</pre>");

                    // Replace Slashes
                    Line = Line.Replace("\\", "&apos;");

                    HtmlLines.AppendLine(Line);
                }
            }

            HtmlLines.AppendLine("</p>");
            HtmlLines.AppendLine("</font>");
            HtmlLines.AppendLine("</BODY>");

            return HtmlLines.ToString();
        }
    }
}
